@@ -1,34 +1,41 @@
 from PyQt5 import uic
-from PyQt5.QtWidgets import (QApplication, QFileDialog, QGraphicsScene)
+from PyQt5.QtWidgets import QApplication, QFileDialog, QGraphicsScene
 from PyQt5.QtCore import QThread, pyqtSignal
+from PyQt5.QtGui import QIcon, QPixmap
 import sys
 import matplotlib.pyplot as plt
 from matplotlib.patches import Circle
-from matplotlib.backends.backend_qt5agg import (FigureCanvasQTAgg as
-                                                FigureCanvas)
+from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 from pathlib import Path
 import logging
 from time import sleep
 import numpy as np
 import os
 
+# hotfix 3.9 MacOS Big Sur bug
+if sys.platform == "darwin":
+    os.environ["QT_MAC_WANTS_LAYER"] = "1"
+
 from .utils import recorder as rec
 from .utils import blockfile as blf
 from .utils import tiffexport as tfe
+from .utils import hspy as hspf
 
 logging.basicConfig(level=logging.DEBUG)
 sys.path.append(".")
 
 logger = logging.getLogger(__name__)
 # import the UI interface
-rawgui, Window = uic.loadUiType(str(Path(__file__).parent.absolute()) +
-                                "/widget_2.ui")
+rawgui, Window = uic.loadUiType(
+    str(Path(__file__).parent.absolute()) + os.sep + "widget.ui"
+)
 
 
 class External(QThread):
     """
     Runs a counter thread.
     """
+
     countChanged = pyqtSignal(int)
     finish = pyqtSignal()
 
@@ -47,6 +54,7 @@ class External(QThread):
 
 class ConnectedWidget(rawgui):
     """Class connecting the gui elements to the back-end functionality"""
+
     def __init__(self, window):
         super().__init__()
         self.window = window
@@ -102,18 +110,17 @@ class ConnectedWidget(rawgui):
         # browsing blo file
         self.pushButton_9.clicked.connect(self.get_blo_path)
         # start blo conversion
-        self.pushButton_5.clicked.connect(self.write_to_blo)
+        self.pushButton_5.clicked.connect(self.write_to_file)
         # select tiff file
         self.pushButton_12.clicked.connect(self.get_tiff_path)
         # export tiff files
         self.pushButton_11.clicked.connect(self.export_tiffs)
-        # self.checkBox_8.stateChanged.connect(self.updateActive)
-        # self.checkBox_3.stateChanged.connect(self.updateActive)
-        # self.checkBox_7.stateChanged.connect(self.updateActive)
-        # self.checkBox_5.stateChanged.connect(self.updateActive)
-        # self.checkBox_4.stateChanged.connect(self.updateActive)
-        # self.checkBox_6.stateChanged.connect(self.updateActive)
-        # self.updateActive()
+        # show cropped region
+        self.pushButton_13.clicked.connect(self.show_cropped_region)
+        # write scan settings (from VBF preview) to hdf5
+        # self.pushButton_write_scan_parameters.clicked.connect(
+        #     self.write_scan_parameters_hdf5
+        # )
 
     def export_preview(self):
         try:
@@ -145,9 +152,9 @@ class ConnectedWidget(rawgui):
             vmax = self.horizontalSlider_2.value()
             mn = self.vbf_data.min()
             mx = self.vbf_data.max()
-            unit = (mx-mn)/100
-            climmin = mn+vmin*unit
-            climmax = mn+(vmax+1)*unit
+            unit = (mx - mn) / 100
+            climmin = mn + vmin * unit
+            climmax = mn + (vmax + 1) * unit
             try:
                 self.vbf_im.set_clim(climmin, climmax)
                 canvas = FigureCanvas(self.fig_vbf)
@@ -155,7 +162,7 @@ class ConnectedWidget(rawgui):
                 scene = QGraphicsScene()
                 scene.addWidget(canvas)
                 self.graphicsView_3.setScene(scene)
-                self.graphicsView_3.fitInView(scene.sceneRect())
+                self.graphicsView_3.fitInView(scene.sceneRect(),)
                 self.repaint_widget(self.graphicsView_3)
             except Exception as e:
                 logger.debug(f"Error: {e}")
@@ -164,18 +171,24 @@ class ConnectedWidget(rawgui):
         pass
 
     def update_final_frame(self):
+        # update x, y crop box values
+        self.spinBox_22.setValue(0)
+        self.spinBox_20.setValue(0)
+        self.spinBox_23.setValue(self.spinBox.value())
+        self.spinBox_21.setValue(self.spinBox_2.value())
+
         if self.checkBox_2.checkState():
             # we use self defined size
             start = self.spinBox_15.value()
-            frms = self.spinBox.value()*self.spinBox_2.value()
-            self.spinBox_16.setValue(start+frms-1)
+            frms = self.spinBox.value() * self.spinBox_2.value()
+            self.spinBox_16.setValue(start + frms - 1)
         else:
             # we use auto-size
             start = self.spinBox_15.value()
             frms = self.lineEdit_11.text()
             try:
                 dim = np.sqrt(int(frms))
-                self.spinBox_16.setValue(start+dim**2-1)
+                self.spinBox_16.setValue(start + dim ** 2 - 1)
             except Exception:
                 self.spinBox_16.setValue(0)
 
@@ -202,14 +215,12 @@ class ConnectedWidget(rawgui):
                 self.statusedit.setText(str(e))
 
     def openFileBrowser(self, fs):
-        path, okpres = QFileDialog.getOpenFileName(caption="Select file",
-                                                   filter=fs)
+        path, okpres = QFileDialog.getOpenFileName(caption="Select file", filter=fs)
         if okpres:
             return str(Path(path))
 
     def saveFileBrowser(self, fs):
-        path, okpres = QFileDialog.getSaveFileName(caption="Select file",
-                                                   filter=fs)
+        path, okpres = QFileDialog.getSaveFileName(caption="Select file", filter=fs)
         if okpres:
             return str(Path(path))
 
@@ -233,13 +244,14 @@ class ConnectedWidget(rawgui):
             "usels": self.checkBox_4.checkState(),
             "lsmin": self.spinBox_7.value(),
             "lsmax": self.spinBox_8.value(),
-            "usecoffset": self.checkBox.checkState()
+            "usecoffset": self.checkBox.checkState(),
+            "bintype": self.radioButton_decimation.isChecked(),  # if True then use decimation otherwise box averaging
         }
         vbfsettings = {
             "calcvbf": self.checkBox_10.checkState(),
             "vbfrad": self.spinBox_12.value(),
             "vbfxoffset": self.spinBox_13.value(),
-            "vbfyoffset": self.spinBox_14.value()
+            "vbfyoffset": self.spinBox_14.value(),
         }
         return path, improc, vbfsettings
 
@@ -247,6 +259,7 @@ class ConnectedWidget(rawgui):
         """Read the first image from the file and create a preview"""
         # read the gui info
         path, improc, vbfsettings = self.read_modsettings()
+        framenum = self.spinBox_17.value()
         # create one image properly processed
         try:
             if not path:
@@ -254,11 +267,12 @@ class ConnectedWidget(rawgui):
             # get and calculate the image. Also get old image and new image
             # size. only change original image if there is none or the path
             # has changed
-            if (self.original_preview is None) or (self.path_preview != path):
-                self.update_line(self.statusedit, "Extracting frame...")
-                self.original_preview = rec.getOriginalPreviewImage(
-                                                path, improc=improc,
-                                                vbfsettings=vbfsettings)
+            # if (self.original_preview is None) or
+            # (self.path_preview != path):
+            self.update_line(self.statusedit, "Extracting frame...")
+            self.original_preview = rec.getOriginalPreviewImage(
+                path, improc=improc, vbfsettings=vbfsettings, frame=framenum
+            )
             # update the path
             self.path_preview = path
             ois = self.original_preview.shape
@@ -266,36 +280,42 @@ class ConnectedWidget(rawgui):
             nis = filterframe.shape
             # check if the VBF aperture fits in the frame
             if vbfsettings["calcvbf"]:
-                midx = nis[1]//2
-                midy = nis[0]//2
+                midx = nis[1] // 2
+                midy = nis[0] // 2
                 xx = vbfsettings["vbfxoffset"]
                 yy = vbfsettings["vbfyoffset"]
                 rr = vbfsettings["vbfrad"]
-                if (midx+xx-rr < 0 or
-                   midx+xx+rr > nis[1] or
-                   midy+yy-rr < 0 or
-                   midy+yy-rr > nis[0]):
-                    raise Exception("Virtual bright field aperture out "
-                                    "of bounds")
+                if (
+                    midx + xx - rr < 0
+                    or midx + xx + rr > nis[1]
+                    or midy + yy - rr < 0
+                    or midy + yy - rr > nis[0]
+                ):
+                    raise Exception("Virtual bright field aperture out " "of bounds")
             # plot the image and the circle over it
             if self.fig_prev is not None:
                 plt.close(self.fig_prev)
-            self.fig_prev = plt.figure(frameon=False,
-                                       figsize=(filterframe.shape[1]/100,
-                                                filterframe.shape[0]/100))
+            self.fig_prev = plt.figure(
+                frameon=False,
+                figsize=(filterframe.shape[1] / 100, filterframe.shape[0] / 100),
+            )
             canvas = FigureCanvas(self.fig_prev)
-            ax = plt.Axes(self.fig_prev, [0., 0., 1., 1.])
+            ax = plt.Axes(self.fig_prev, [0.0, 0.0, 1.0, 1.0])
             ax.set_axis_off()
             self.fig_prev.add_axes(ax)
             ax.imshow(filterframe, cmap="Greys_r")
             if vbfsettings["calcvbf"]:
                 xoff = vbfsettings["vbfxoffset"]
                 yoff = vbfsettings["vbfyoffset"]
-                circ = Circle((filterframe.shape[1]//2+xoff,
-                               filterframe.shape[0]//2+yoff),
-                              vbfsettings["vbfrad"],
-                              color="red",
-                              alpha=0.5)
+                circ = Circle(
+                    (
+                        filterframe.shape[1] // 2 + xoff,
+                        filterframe.shape[0] // 2 + yoff,
+                    ),
+                    vbfsettings["vbfrad"],
+                    color="red",
+                    alpha=0.5,
+                )
                 ax.add_patch(circ)
             canvas.draw()
             scene = QGraphicsScene()
@@ -304,8 +324,10 @@ class ConnectedWidget(rawgui):
             self.graphicsView.fitInView(scene.sceneRect())
             self.repaint_widget(self.graphicsView)
             self.update_line(self.statusedit, "Succesfully created preview.")
-            self.update_line(self.lineEdit_8, f"Original: {ois[0]}x{ois[1]}. "
-                                              f"New: {nis[0]}x{nis[1]}.")
+            self.update_line(
+                self.lineEdit_8,
+                f"Original: {ois[0]}x{ois[1]}. " f"New: {nis[0]}x{nis[1]}.",
+            )
         except Exception as e:
             self.update_line(self.statusedit, f"Error: {e}")
             # empty the preview
@@ -329,8 +351,7 @@ class ConnectedWidget(rawgui):
         # open a savefile browser
         try:
             # read the gui info
-            (self.inpath, self.improc,
-             self.vbfsettings) = self.read_modsettings()
+            (self.inpath, self.improc, self.vbfsettings) = self.read_modsettings()
             if not self.inpath:
                 raise Exception("A TVIPS file must be selected!")
             self.oupath = self.saveFileBrowser("HDF5 (*.hdf5)")
@@ -354,7 +375,13 @@ class ConnectedWidget(rawgui):
     def get_blo_path(self):
         # open a savefile browser
         try:
-            self.oupath = self.saveFileBrowser("BLO (*.blo)")
+            filetype = self.comboBox.currentText()
+            if ".blo" == filetype:
+                self.oupath = self.saveFileBrowser("BLO (*.blo)")
+            elif ".hspy" == filetype:
+                self.oupath = self.saveFileBrowser("HSPY (*.hspy)")
+            else:
+                raise ValueError("Unrecognized filetype")
             if not self.oupath:
                 raise Exception("No valid BLO file path selected")
             self.lineEdit_7.setText(self.oupath)
@@ -374,10 +401,16 @@ class ConnectedWidget(rawgui):
     def done(self):
         self.window.setEnabled(True)
 
+    @property
+    def image_range(self):
+        if self.checkBox_13.checkState():
+            return self.spinBox_19.value(), self.spinBox_18.value()
+        else:
+            return None, None
+
     def write_to_hdf5(self):
         try:
-            (self.inpath, self.improc,
-             self.vbfsettings) = self.read_modsettings()
+            (self.inpath, self.improc, self.vbfsettings) = self.read_modsettings()
             if not self.inpath:
                 raise Exception("A TVIPS file must be selected!")
             self.oupath = self.lineEdit_2.text()
@@ -389,10 +422,21 @@ class ConnectedWidget(rawgui):
             opath = self.oupath
             improc = self.improc
             vbfsettings = self.vbfsettings
-            self.get_thread = rec.Recorder(path,
-                                           improc=improc,
-                                           vbfsettings=vbfsettings,
-                                           outputpath=opath)
+            start_frame, end_frame = self.image_range
+            self.get_thread = rec.Recorder(
+                path,
+                improc=improc,
+                vbfsettings=vbfsettings,
+                outputpath=opath,
+                imrange=(start_frame, end_frame),
+                calcmax=self.checkBox_maxiumum_image.isChecked(),  # options kwarg
+                calcave=self.checkBox_average_image.isChecked(),  # options kwarg
+                refine_center=(
+                    self.checkBox_refine_center.isChecked(),
+                    self.spinBox_refine_center_diameter.value(),
+                    self.spinBox_refine_center_sigma.value(),
+                ),
+            )
             self.get_thread.increase_progress.connect(self.increase_progbar)
             self.get_thread.finish.connect(self.done_hdf5export)
             self.get_thread.start()
@@ -403,8 +447,7 @@ class ConnectedWidget(rawgui):
     def done_hdf5export(self):
         self.window.setEnabled(True)
         # also update lines in the second pannel
-        self.update_line(self.statusedit,
-                         "Succesfully exported to HDF5")
+        self.update_line(self.statusedit, "Succesfully exported to HDF5")
         # don't auto update, the gui may be before the file exists
         # self.update_line(self.lineEdit_4, self.lineEdit_2.text())
 
@@ -419,6 +462,9 @@ class ConnectedWidget(rawgui):
                 self.update_line(self.lineEdit_3, "?")
             if star is not None:
                 self.update_line(self.lineEdit_5, str(star))
+                logger.info(f"start: {star}")
+                self.spinBox_15.setValue(star)
+                self.checkBox_11.setChecked(True)
             else:
                 self.update_line(self.lineEdit_5, "?")
             if en is not None:
@@ -430,12 +476,18 @@ class ConnectedWidget(rawgui):
             else:
                 self.update_line(self.lineEdit_11, "?")
             if dim is not None:
-                self.update_line(self.lineEdit_12,
-                                 f"{str(int(dim))}x{str(int(dim))}")
+                if isinstance(dim, (list, tuple)):
+                    self.update_line(self.lineEdit_12, str(dim))
+                    self.spinBox.setValue(dim[0])
+                    self.spinBox_2.setValue(dim[1])
+                    self.checkBox_2.setChecked(True)
+                else:
+                    self.update_line(
+                        self.lineEdit_12, f"{str(int(dim))}x{str(int(dim))}"
+                    )
             else:
                 self.update_line(self.lineEdit_12, "?")
-            self.update_line(self.lineEdit_13,
-                             f"{str(imdimx)}x{str(imdimy)}")
+            self.update_line(self.lineEdit_13, f"{str(imdimx)}x{str(imdimy)}")
             self.update_final_frame()
             f.close()
         except Exception as e:
@@ -481,29 +533,36 @@ class ConnectedWidget(rawgui):
             else:
                 snakescan = False
             # calculate the image
-            logger.debug(f"We try to create a VBF image with data: "
-                         f"S.F. {start_frame}, E.F. {end_frame}, "
-                         f"Dims: x {sdimx} y {sdimy},"
-                         f"hyst: {hyst}")
-            self.vbf_data = f.get_vbf_image(sdimx, sdimy, start_frame,
-                                            end_frame, hyst, snakescan)
+            logger.debug(
+                f"We try to create a VBF image with data: "
+                f"S.F. {start_frame}, E.F. {end_frame}, "
+                f"Dims: x {sdimx} y {sdimy},"
+                f"hyst: {hyst}"
+            )
+            self.vbf_data = f.get_vbf_image(
+                sdimx, sdimy, start_frame, end_frame, hyst, snakescan
+            )
             logger.debug("Succesfully created the VBF array")
             # save the settings for later storage
-            self.vbf_sets = {"start_frame": start_frame,
-                             "end_frame": end_frame,
-                             "scan_dim_x": sdimx,
-                             "scan_dim_y": sdimy,
-                             "hysteresis": hyst,
-                             "winding_scan": snakescan}
+            self.vbf_sets = {
+                "start_frame": start_frame,
+                "end_frame": end_frame,
+                "scan_dim_x": sdimx,
+                "scan_dim_y": sdimy,
+                "hysteresis": hyst,
+                "winding_scan": snakescan,
+            }
+
             # plot the image and store it for further use. First close prior
             # image
             if self.fig_vbf is not None:
                 plt.close(self.fig_vbf)
-            self.fig_vbf = plt.figure(frameon=False,
-                                      figsize=(self.vbf_data.shape[1]/100,
-                                               self.vbf_data.shape[0]/100))
+            self.fig_vbf = plt.figure(
+                frameon=False,
+                figsize=(self.vbf_data.shape[1] / 100, self.vbf_data.shape[0] / 100),
+            )
             canvas = FigureCanvas(self.fig_vbf)
-            ax = plt.Axes(self.fig_vbf, [0., 0., 1., 1.])
+            ax = plt.Axes(self.fig_vbf, [0.0, 0.0, 1.0, 1.0])
             ax.set_axis_off()
             self.fig_vbf.add_axes(ax)
             self.vbf_im = ax.imshow(self.vbf_data, cmap="plasma")
@@ -518,11 +577,14 @@ class ConnectedWidget(rawgui):
             yshap, xshap = self.vbf_data.shape
             self.update_line(self.lineEdit_10, f"Size: {xshap}x{yshap}.")
             f.close()
+
+            # add settings to hdf5 file, do this after file close
+            rec.write_scan_parameters_hdf5(path_hdf5, **self.vbf_sets)
         except Exception as e:
             self.update_line(self.statusedit, f"Error: {e}")
 
-    def write_to_blo(self):
-        """Export the file to blo in an incremental way"""
+    def write_to_file(self):
+        """Export the data to a specific file format in an incremental way"""
         path_hdf5 = self.lineEdit_4.text()
         path_blo = self.lineEdit_7.text()
         try:
@@ -530,7 +592,7 @@ class ConnectedWidget(rawgui):
             if not path_hdf5:
                 raise Exception("No valid HDF5 file selected!")
             if not path_blo:
-                raise Exception("No valid blo file selected!")
+                raise Exception("No valid export file path selected!")
             # try to read the info from the file
             f = rec.hdf5Intermediate(path_hdf5)
             # none or 0 means default
@@ -539,7 +601,7 @@ class ConnectedWidget(rawgui):
             sdimx = None
             sdimy = None
             hyst = 0
-            snakescan = True
+            snakescan = False
             # overwrite standard info depending on gui
             if self.checkBox_2.checkState():
                 # use custom scanning
@@ -557,19 +619,56 @@ class ConnectedWidget(rawgui):
                 snakescan = True
             else:
                 snakescan = False
+            # we set the scale of scan and dp
+            scan_scale = self.doubleSpinBox_2.value()
+            dp_scale = self.doubleSpinBox_3.value()
             # calculate the image
-            logger.debug(f"We try to create a blo file with data: "
-                         f"S.F. {start_frame}, E.F. {end_frame}, "
-                         f"Dims: x {sdimx} y {sdimy},"
-                         f"hyst: {hyst}, snakescan: {snakescan}")
+            filetyp = self.comboBox.currentText()
+            logger.debug(
+                f"We try to create a {filetyp} file with data: "
+                f"S.F. {start_frame}, E.F. {end_frame}, "
+                f"Dims: x {sdimx} y {sdimy},"
+                f"hyst: {hyst}, snakescan: {snakescan}"
+            )
             logger.debug("Calculating shape and indexes")
-            shape, indexes = f.get_blo_export_data(sdimx, sdimy,
-                                                   start_frame,
-                                                   end_frame, hyst, snakescan)
+
+            # check crop desired
+            if self.checkBox_apply_crop.isChecked():
+                # check crop valid
+                if self.check_crop_limits() is None:
+                    self.update_line(
+                        self.statusedit,
+                        f"File not exported, please check cropping limits.",
+                    )
+                    return
+                # get crop
+                crop = self.get_crop_limits()
+            else:
+                crop = None
+
+            shape, indexes = f.get_blo_export_data(
+                sdimx, sdimy, start_frame, end_frame, hyst, snakescan, crop=crop
+            )
             logger.debug(f"Shape: {shape}")
-            logger.debug("Starting to write blo file")
-            self.update_line(self.statusedit, "Writing blo file...")
-            self.get_thread = blf.bloFileWriter(f, path_blo, shape, indexes)
+            logger.debug(f"Starting to write {filetyp} file")
+            self.update_line(self.statusedit, f"Writing {filetyp} file...")
+            if filetyp == ".blo":
+                self.get_thread = blf.bloFileWriter(
+                    f,
+                    path_blo,
+                    shape,
+                    indexes,
+                    scan_scale,
+                    dp_scale,
+                    # if rescale button is checked do rescale, otherwise no rescaling selected
+                    rescale=self.checkBox_rescale.isChecked(),
+                )
+            elif filetyp == ".hspy":
+                self.get_thread = hspf.hspyFileWriter(
+                    f, path_blo, shape, indexes, scan_scale, dp_scale
+                )
+            else:
+                raise NotImplementedError("Unrecognized file type")
             self.get_thread.increase_progress.connect(self.increase_progbar)
             self.get_thread.finish.connect(self.done_bloexport)
             self.get_thread.start()
@@ -580,8 +679,7 @@ class ConnectedWidget(rawgui):
     def done_bloexport(self):
         self.window.setEnabled(True)
         # also update lines in the second pannel
-        self.update_line(self.statusedit,
-                         "Succesfully exported to blo")
+        self.update_line(self.statusedit, "Succesfully exported to file")
 
     def export_tiffs(self):
         path_hdf5 = self.lineEdit_4.text()
@@ -607,7 +705,7 @@ class ConnectedWidget(rawgui):
                 raise Exception("Unexpected dtype")
             if tot_frames <= last_frame:
                 raise Exception("Frames are out of range")
-            frames = np.arange(first_frame, last_frame+1)
+            frames = np.arange(first_frame, last_frame + 1)
             self.update_line(self.statusedit, "Exporting to tiff files...")
             self.get_thread = tfe.TiffFileWriter(f, frames, dtype, pre, fin)
             self.get_thread.increase_progress.connect(self.increase_progbar)
@@ -620,8 +718,7 @@ class ConnectedWidget(rawgui):
     def done_tiffexport(self):
         self.window.setEnabled(True)
         # also update lines in the second pannel
-        self.update_line(self.statusedit,
-                         "Succesfully exported Tiff files")
+        self.update_line(self.statusedit, "Succesfully exported Tiff files")
 
     def increase_progbar(self, value):
         self.progressBar.setValue(value)
@@ -630,16 +727,110 @@ class ConnectedWidget(rawgui):
         self.window.hide()
         self.window.show()
 
+    def remove_cropped_region(self):
+        # try to remove if it is there
+        label = "crop_rect"
+        try:
+            # see if rectangle already plotted and just update
+            index = [i.get_label() for i in self.fig_vbf.axes[0].patches].index(label)
+            rect = self.fig_vbf.axes[0].patches[index]
+            rect.remove()
+            self.fig_vbf.canvas.draw()
+        except ValueError:
+            logger.info("No crop rectange drawn.")
+
+    def get_crop_limits(self):
+        xmin = self.spinBox_22.value()
+        xmax = self.spinBox_23.value()
+        ymin = self.spinBox_20.value()
+        ymax = self.spinBox_21.value()
+
+        return xmin, xmax, ymin, ymax
+
+    def check_crop_limits(self):
+        xmin, xmax, ymin, ymax = self.get_crop_limits()
+
+        # check scan boundaries, returns None if any test fails
+        if (
+            xmin < 0
+            or ymin < 0
+            or not xmax <= self.spinBox.value()
+            or not ymax <= self.spinBox_2.value()
+        ):
+            self.update_line(
+                self.statusedit, "Crop limits should be within scan bounds."
+            )
+            self.remove_cropped_region()
+            return
+
+        # check cropping dimension > 0
+        if not xmax - xmin > 0:
+            self.update_line(
+                self.statusedit, f"Crop dimension incorrect. Crop x: {xmax - xmin}.",
+            )
+            self.remove_cropped_region()
+            return
+        if not ymax - ymin > 0:
+            self.update_line(
+                self.statusedit, f"Crop dimension incorrect. Crop y: {ymax - ymin}.",
+            )
+            self.remove_cropped_region()
+            return
+
+        # return non-None if passes checks
+        return True
+
+    def show_cropped_region(self):
+        if self.check_crop_limits() is None:
+            return
+
+        # check for validity of cropped region
+        xmin, xmax, ymin, ymax = self.get_crop_limits()
+
+        if self.fig_vbf is None:
+            self.update_line(
+                self.statusedit, "No VBF figure plotted yet.",
+            )
+            return
+
+        label = "crop_rect"
+        try:
+            # see if rectangle already plotted and just update
+            index = [i.get_label() for i in self.fig_vbf.axes[0].patches].index(label)
+            rect = self.fig_vbf.axes[0].patches[index]
+            logger.info("Updating rectangle.")
+            rect.set_height(ymax - ymin)
+            rect.set_width(xmax - xmin)
+            rect.set_x(xmin)
+            rect.set_y(ymin)
+        except ValueError:
+            logger.info("Plotting rectangle.")
+            rect = plt.Rectangle(
+                (xmin, ymin),
+                xmax - xmin,
+                ymax - ymin,
+                fill=False,
+                ec="k",
+                ls="dashed",
+                label=label,
+            )
+            self.fig_vbf.axes[0].add_patch(rect)
+        self.fig_vbf.canvas.draw()
+
+        self.update_line(
+            self.statusedit, f"Crop dimensions (x, y): ({xmax - xmin}, {ymax - ymin}).",
+        )
+
 
 def main():
     app = QApplication([])
     window = Window()
     _ = ConnectedWidget(window)
     window.setWindowTitle("TVIPS converter")
+    # window.setWindowIcon(
+    #     QIcon(QPixmap(os.path.join(os.path.dirname(__file__), "Icon 128.png")))
+    # )
     window.show()
-    # form.lineEdit.setText("/Volumes/Elements/200309-2F/"
-    #                       "rec_20200309_113857_000.tvips")
-    # form.lineEdit_2.setText("/Users/nielscautaerts/Desktop/stream_2.hdf5")
     app.exec_()
 
 
